%% Hanging Catenary problem discretisation error
% Lindon Roberts, 2019

clear, close all

% Parameters
m = 1; % mass
g = 9.8; % gravity
L = 1; % beam lengths
gamma = 0.95; % horizontal bridge length / total beam lengths

ns = (5:5:50);
es = zeros(1,numel(ns));

for i=1:numel(ns)
    n = ns(i);
    
    % Get function for analytic solution, to see discretisation error
    true_soln = @(x)analytic_solution(x, m, g, n, L, gamma);

    % Choose a sensible initial guess
    xcoords0 = linspace(0,gamma*(n+1)*L,n+2)'; % evenly distributed
    ycoords0 = zeros(n+2,1);
    zcoords0 = zeros(n+2,1);
    x0 = [xcoords0; ycoords0; zcoords0];

    % Create anonymous functions for the objective and constraints
    obj = @(x)objfun(x, m, g);
    nonlcon = @(x)nlconstr(x, L);
    A = []; b = []; % no linear inequality constraints
    Aeq = []; beq = []; % no linear equality constraints

    % Bound constraints lb <= x <= ub
    lb = -1e20*ones(3*(n+2), 1); % default to large negative value
    ub = 1e20*ones(3*(n+2), 1); % default to large positive value
    lb(1) = 0; ub(1) = 0; % 0 <= x0 <= 0
    lb(n+3) = 0; ub(n+3) = 0; % 0 <= y0 <= 0
    lb(2*n+5) = 0; ub(2*n+5) = 0; % 0 <= z0 <= 0
    lb(n+2) = gamma*(n+1)*L; ub(n+2) = gamma*(n+1)*L; % gamma*(n+1)*L <= x_{n+1} <= gamma*(n+1)*L
    lb(2*n+4) = 0; ub(2*n+4) = 0; % 0 <= y_{n+1} <= 0
    lb(3*n+6) = 0; ub(3*n+6) = 0; % 0 <= z_{n+1} <= 0

    % Set options
    options = optimoptions('fmincon'); % start with defaults
    options = optimoptions(options, 'Algorithm', 'interior-point');
    options = optimoptions(options, 'Display', 'off');
    options = optimoptions(options, 'MaxFunctionEvaluations', 3000);
    options = optimoptions(options, 'MaxIterations', 1000);
    options = optimoptions(options, 'ConstraintTolerance', 1e-6);
    options = optimoptions(options, 'OptimalityTolerance', 1e-6);
    options = optimoptions(options, 'StepTolerance', 1e-10);
    options = optimoptions(options, 'SpecifyConstraintGradient', true);
    options = optimoptions(options, 'SpecifyObjectiveGradient', true);
    options = optimoptions(options, 'HessianFcn', @hessfun);

    % Call Matlab solver
    [xmin,fmin,exitflag,output,lambda,grad,hessian] = fmincon(obj,x0,A,b,Aeq,beq,lb,ub,nonlcon,options);
    
    % Sort out results
    xcoords = xmin(1:n+2);
    ycoords = xmin(n+3:2*n+4);
    zcoords = xmin(2*n+5:3*n+6);

    ycoords_true = true_soln(xcoords);
    
    es(i) = norm(ycoords - ycoords_true);
end

plot(ns, es, 'b.-', 'LineWidth', 2, 'MarkerSize', 20);
xlabel('n');
ylabel('Error (discrete v continuous)');
