function [H] = hessfun(x, lambda)
    % Hessian of Lagrangian for Hanging Catenary problem
    % Lindon Roberts, 2019
    
    % Call as:
    %   H  = hessfun(x, lambda);
    % where lambda.ineqnonlin and lambda.eqnonlin are the Lagrange
    % multipliers for the (in)equality constraints, as per nlconstr.m
    
    % x is large vector of size 3*(n+2)
    n = (numel(x)-6)/3;
    
    % The first n+2 elements are the x-coordinates
    % The next n+2 elements are the y-coordinates
    % The last n+2 elements are the z-coordinates
    xcoords = x(1:n+2);
    ycoords = x(n+3:2*n+4);
    zcoords = x(2*n+5:end);
    
    % Hessian of Lagrangian is:
    %   H = hess_objfun + sum( lambda_i * hess_constraint_i )
    % where the sum is over all (nonlinear) constraints
    % Note: the Lagrangian includes all constraints, but the Hessian only
    %       needs the nonlinear constraints (linear constraints have zero
    %       Hessian).
    
    % Hessian of the objective function - zero since it is linear
    H = zeros(3*(n+2));
    
    % Only nonzero bits of the Hessian come from lambda.eqnonlin
    nceq = n+1;
    for i=1:nceq
        % Hessian for this constraint
        Hceq_i = zeros(3*(n+2));
        
        % Diagonal entries
        Hceq_i(i,i)             = 2; % x(i)
        Hceq_i(i+1,i+1)         = 2; % x(i+1)
        Hceq_i(n+2+i,n+2+i)     = 2; % y(i)
        Hceq_i(n+3+i,n+3+i)     = 2; % y(i+1)
        Hceq_i(2*n+4+i,2*n+4+i) = 2; % z(i)
        Hceq_i(2*n+5+i,2*n+5+i) = 2; % z(i+1)
        
        % Off-diagonal entries
        Hceq_i(i,i+1)           = -2; % x(i), x(i+1)
        Hceq_i(i+1,i)           = -2; % x(i+1), x(i)
        Hceq_i(n+2+i,n+3+i)     = -2; % y(i), y(i+1)
        Hceq_i(n+3+i,n+2+i)     = -2; % y(i+1), y(i)
        Hceq_i(2*n+4+i,2*n+5+i) = -2; % z(i), z(i+1)
        Hceq_i(2*n+5+i,2*n+4+i) = -2; % z(i+1), z(i)
        
        % Increment full Hessian of Lagrangian
        H = H + lambda.eqnonlin(i) * Hceq_i;
    end
    
    % Make the Hessian sparse to save computation in the solver
    % (in practice, we should define H to be sparse in the first place)
    H = sparse(H);
end