%% Solve Hanging Catenary problem (using Matlab Optimization Toolbox) - template script
% Lindon Roberts, 2019

clear, close all

% Parameters
m = 1; % mass
g = 9.8; % gravity
n = 16; % number of beams = n+1
L = 1; % beam lengths
gamma = 0.95; % horizontal bridge length / total beam lengths

% Our convention for input variables is:
%   x is large vector of size 3*(n+2)
%   The first n+2 elements are the x-coordinates: x0, ..., xnp1 = x(1:n+2)
%   The next n+2 elements are the y-coordinates: y0, ..., ynp1 = x(n+3:2*n+4)
%   The last n+2 elements are the z-coordinates: z0, ..., znp1 = x(2*n+5:3*n+6)

% Choose a sensible initial guess (allowed to be infeasible)
xcoords0 = linspace(0,gamma*(n+1)*L,n+2)'; % evenly distributed
ycoords0 = zeros(n+2,1);
zcoords0 = zeros(n+2,1);
x0 = [xcoords0; ycoords0; zcoords0];

% Create anonymous functions for the objective and constraints
obj = @(x)objfun(x, m, g);
nonlcon = @(x)nlconstr(x, L);
A = []; b = []; % no linear inequality constraints
Aeq = []; beq = []; % no linear equality constraints

% Bound constraints lb <= x <= ub
lb = -1e20*ones(3*(n+2), 1); % default to large negative value
ub = 1e20*ones(3*(n+2), 1); % default to large positive value
lb(1) = 0; ub(1) = 0; % 0 <= x0 <= 0
lb(n+3) = 0; ub(n+3) = 0; % 0 <= y0 <= 0
lb(2*n+5) = 0; ub(2*n+5) = 0; % 0 <= z0 <= 0
lb(n+2) = gamma*(n+1)*L; ub(n+2) = gamma*(n+1)*L; % gamma*(n+1)*L <= x_{n+1} <= gamma*(n+1)*L
lb(2*n+4) = 0; ub(2*n+4) = 0; % 0 <= y_{n+1} <= 0

% Set options
options = optimoptions('fmincon'); % start with defaults
% Set options with
%   options = optimoptions(options, 'OptionToChange', value_to_use);
options = optimoptions(options, 'Algorithm', 'interior-point');
options = optimoptions(options, 'Display', 'iter');
options = optimoptions(options, 'SpecifyConstraintGradient', true);
options = optimoptions(options, 'SpecifyObjectiveGradient', true);
%options = optimoptions(options, 'HessianFcn', @hessfun);

% Call Matlab solver
[xmin,fmin,exitflag,output,lambda,grad,hessian] = fmincon(obj,x0,A,b,Aeq,beq,lb,ub,nonlcon,options);

% Split out results into (x,y,z) vectors
xcoords = xmin(1:n+2);
ycoords = xmin(n+3:2*n+4);
zcoords = xmin(2*n+5:3*n+6);

% TODO: Inspect the outputs and understand what information you have



%==============================
% Plot our solution against the analytic solution (n->infinity), if desired
% Do not try to derive the analytic solution in the lab
%true_soln = @(x)analytic_solution(x, m, g, n, L, gamma);
%xfull = linspace(xcoords(1), xcoords(end), 100); % for plotting
%yfull = true_soln(xfull);
%ycoords_true = true_soln(xcoords);
%plot(xcoords, ycoords, 'b-', 'LineWidth', 2);
%hold on
%plot(xfull, yfull, 'r--', 'LineWidth', 2);
%hold off
%==============================
