function varargout = nlconstr(x, L)
    % Hanging Catenary constraints function
    % Lindon Roberts, 2019
    
    % Call as:
    %   [c, ceq]           = nlconstr(x, m, g);  % constraint values only
    %   [c, ceq, Gc, Gceq] = nlconstr(x, m, g);  % constraint values and Jacobians
    
    if nargout ~= 2 && nargout ~= 4
        error('Invalid number of output arguments');
    end
    
    % x is large vector of size 3*(n+2)
    n = (numel(x)-6)/3;
    
    % The first n+2 elements are the x-coordinates
    % The next n+2 elements are the y-coordinates
    % The last n+2 elements are the z-coordinates
    xcoords = x(1:n+2);
    ycoords = x(n+3:2*n+4);
    zcoords = x(2*n+5:end);
    
    % Inequality constraints c(x)<=0
    c = [];
    varargout{1} = c;
    
    % Equality constraints ceq(x)=0
    nceq = n+1; % number of equality constraints
    ceq = zeros(nceq,1);
    for i=1:nceq
        ceq(i) = (xcoords(i)-xcoords(i+1)).^2 + (ycoords(i)-ycoords(i+1)).^2 + (zcoords(i) - zcoords(i+1)).^2 - L.^2;
    end
    varargout{2} = ceq;
    
    if nargout > 2
        % Jacobian for inequality constraints
        Gc = [];
        varargout{3} = Gc;
        
        % Jacobian for equality constraints
        Gceq = zeros(3*(n+2),nceq);
        for i=1:nceq
            % For each ceq(i), the gradient is Gceq(:,i)
            Gceq(i,i)       =  2*(xcoords(i) - xcoords(i+1)); % x(i)
            Gceq(i+1,i)     = -2*(xcoords(i) - xcoords(i+1)); % x(i+1)
            Gceq(n+2+i,i)   =  2*(ycoords(i) - ycoords(i+1)); % y(i)
            Gceq(n+3+i,i)   = -2*(ycoords(i) - ycoords(i+1)); % y(i+1)
            Gceq(2*n+4+i,i) =  2*(zcoords(i) - zcoords(i+1)); % z(i)
            Gceq(2*n+5+i,i) = -2*(zcoords(i) - zcoords(i+1)); % z(i+1)
        end
        % Make the Jacobian sparse to save computation in the solver
        % (in practice, we should define Gceq to be sparse in the first place)
        Gceq = sparse(Gceq);
        varargout{4} = Gceq;
    end
end
