%% Solve Hanging Catenary problem (using KNitro)
% Lindon Roberts, 2019

clear, close all

% Probably need to add environment variables
% export LD_LIBRARY_PATH="/path/to/knitro-11.1.0-z-Linux-Intel17-64:$LD_LIBRARY_PATH"
addpath('knitro-11.1.0-z-Linux-Intel17-64/knitromatlab');

% Parameters
m = 1; % mass
g = 9.8; % gravity
n = 16; % number of beams = n+1
L = 1; % beam lengths
gamma = 0.95; % horizontal bridge length / total beam lengths

% Get function for analytic solution, to see discretisation error
true_soln = @(x)analytic_solution(x, m, g, n, L, gamma);

% Our convention for input variables is:
%   x is large vector of size 3*(n+2)
%   The first n+2 elements are the x-coordinates: x0, ..., xnp1 = x(1:n+2)
%   The next n+2 elements are the y-coordinates: y0, ..., ynp1 = x(n+3:2*n+4)
%   The last n+2 elements are the z-coordinates: z0, ..., znp1 = x(2*n+5:3*n+6)

% Choose a sensible initial guess (allowed to be infeasible)
xcoords0 = linspace(0,gamma*(n+1)*L,n+2)'; % evenly distributed
ycoords0 = zeros(n+2,1);
zcoords0 = zeros(n+2,1);
x0 = [xcoords0; ycoords0; zcoords0];
%x0 = x0 + 100;

% Create anonymous functions for the objective and constraints
obj = @(x)objfun(x, m, g);
nonlcon = @(x)nlconstr(x, L);
A = []; b = []; % no linear inequality constraints
Aeq = []; beq = []; % no linear equality constraints

% Bound constraints lb <= x <= ub
lb = -1e20*ones(3*(n+2), 1); % default to large negative value
ub = 1e20*ones(3*(n+2), 1); % default to large positive value
lb(1) = 0; ub(1) = 0; % 0 <= x0 <= 0
lb(n+3) = 0; ub(n+3) = 0; % 0 <= y0 <= 0
lb(2*n+5) = 0; ub(2*n+5) = 0; % 0 <= z0 <= 0
lb(n+2) = gamma*(n+1)*L; ub(n+2) = gamma*(n+1)*L; % gamma*(n+1)*L <= x_{n+1} <= gamma*(n+1)*L
lb(2*n+4) = 0; ub(2*n+4) = 0; % 0 <= y_{n+1} <= 0
%lb(3*n+6) = 0; ub(3*n+6) = 0; % 0 <= z_{n+1} <= 0

% Set options
options = optimset('Algorithm','interior-point');
options = optimset(options, 'Display', 'iter');
options = optimset(options, 'MaxIter', 1000);
options = optimset(options, 'TolCon', 1e-6);
options = optimset(options, 'TolFun', 1e-6);
options = optimset(options, 'TolX', 1e-10);
options = optimset(options, 'GradObj', 'on');
options = optimset(options, 'GradConstr', 'on');
options = optimset(options, 'Hessian', 'user-supplied', 'HessFcn', @hessfun);


% Call KNitro solver
[xmin,fmin,exitflag,output,lambda,grad,hessian] = knitromatlab(obj,x0,A,b,Aeq,beq,lb,ub,nonlcon,[],options,[]);
fprintf('=================================\n');
fprintf('Got fmin = %g after %g iterations and %g objective evaluations\n', fmin, output.iterations, output.funcCount);
fprintf('First order optimality = %g, constraint violation = %g\n', output.firstorderopt, output.constrviolation);
fprintf('Norm(Gradient of objective) = %g\n', norm(grad));

% Sort out results
xcoords = xmin(1:n+2);
ycoords = xmin(n+3:2*n+4);
zcoords = xmin(2*n+5:3*n+6);

xfull = linspace(xcoords(1), xcoords(end), 100); % for plotting
yfull = true_soln(xfull);
ycoords_true = true_soln(xcoords);


% Check feasibility
[c_min, ceq_min] = nonlcon(xmin);
if false
    fprintf('=================================\n');
    fprintf('Feasibility: want ceq(xmin) = 0, got ||ceq(xmin)|| = %g\n', norm(ceq_min));
    fprintf('Lagrange multipliers for nonlinear constraints\n');
    disp(lambda.eqnonlin');
    fprintf('Lagrange multipliers for lower bounds\n');
    disp(lambda.lower');
    fprintf('Lagrange multipliers for upper bounds\n');
    disp(lambda.upper');
    fprintf('=================================\n');
end

% Plot our solution against the analytic solution (n->infinity), if desired
%plot(xcoords, ycoords, 'b-', 'LineWidth', 2);
%hold on
%plot(xfull, yfull, 'r--', 'LineWidth', 2);
%hold off
